@@ -1,7 +1,7 @@
 #------------------------------------------------------------------------------
 # File:         CanonRaw.pm
 #
-# Description:  Definitions for Canon CRW file information
+# Description:  Read Canon RAW (CRW) meta information
 #
 # Revisions:    11/25/2003 - P. Harvey Created
 #               12/02/2003 - P. Harvey Completely reworked and figured out many
@@ -19,8 +19,9 @@ use strict;
 use vars qw($VERSION $AUTOLOAD %crwTagFormat);
 use Image::ExifTool qw(:DataAccess);
 use Image::ExifTool::Exif;
+use Image::ExifTool::Canon;
 
-$VERSION = '1.27';
+$VERSION = '1.39';
 
 sub WriteCRW($$);
 sub ProcessCanonRaw($$$);
@@ -52,7 +53,11 @@ sub BuildMakerNotes($$$$$$);
     CHECK_PROC => \&CheckCanonRaw,
     WRITABLE => 1,
     0x0000 => 'NullRecord', #3
-    0x0001 => 'FreeBytes', #3
+    0x0001 => { #3
+        Name => 'FreeBytes',
+        Format => 'undef',
+        Binary => 1,
+    },
     0x0032 => { Name => 'CanonColorInfo1', Writable => 0 },
     0x0805 => [
         # this tag is found in more than one directory...
@@ -104,6 +109,12 @@ sub BuildMakerNotes($$$$$$);
     },
     0x1016 => 'ReleaseSetting', #3
     0x101c => 'BaseISO', #3
+    0x1028=> { #PH
+        Name => 'CanonFlashInfo',
+        Writable => 'int16u',
+        Count => 4,
+        Unknown => 1,
+    },
     0x1029 => {
         Name => 'CanonFocalLength',
         Writable => 0,
@@ -130,36 +141,45 @@ sub BuildMakerNotes($$$$$$);
         Name => 'SensorInfo',
         Writable => 0,
         SubDirectory => {
-            TagTable => 'Image::ExifTool::CanonRaw::SensorInfo',
+            TagTable => 'Image::ExifTool::Canon::SensorInfo',
         },
     },
-    # this tag has only be verified for the 10D in CRW files, but this
-    # is the way it works for the other models with JPG files...
+    # this tag has only be verified for the 10D in CRW files, but the D30 and D60
+    # also produce CRW images and have CustomFunction information in their JPEG's
     0x1033 => [
         {
-            Condition => '$self->{CameraModel} =~ /10D/',
-            Name => 'CanonCustomFunctions10D',
-            Writable => 0,
+            Name => 'CustomFunctions10D',
+            Condition => '$self->{CameraModel} =~ /EOS 10D/',
             SubDirectory => {
+                Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
                 TagTable => 'Image::ExifTool::CanonCustom::Functions10D',
             },
         },
         {
-            Condition => '$self->{CameraModel} =~ /20D/',
-            Name => 'CanonCustomFunctions20D',
-            Writable => 0,
+            Name => 'CustomFunctionsD30',
+            Condition => '$self->{CameraModel} =~ /EOS D30\b/',
             SubDirectory => {
-                TagTable => 'Image::ExifTool::CanonCustom::Functions20D',
+                Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
+                TagTable => 'Image::ExifTool::CanonCustom::FunctionsD30',
             },
         },
         {
-            # assume everything else is a D30/D60
-            Name => 'CanonCustomFunctionsD30',
-            Writable => 0,
+            Name => 'CustomFunctionsD60',
+            Condition => '$self->{CameraModel} =~ /EOS D60\b/',
             SubDirectory => {
+                # the stored size in the D60 apparently doesn't include the size word:
+                Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size-2,$size)',
+                # (D60 custom functions are basically the same as D30)
                 TagTable => 'Image::ExifTool::CanonCustom::FunctionsD30',
             },
         },
+        {
+            Name => 'CustomFunctionsUnknown',
+            SubDirectory => {
+                Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
+                TagTable => 'Image::ExifTool::CanonCustom::FuncsUnknown',
+            },
+        },
     ],
     0x1038 => {
         Name => 'CanonPictureInfo',
@@ -168,14 +188,28 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::Canon::PictureInfo',
         },
     },
+    0x1093 => {
+        Name => 'CanonFileInfo',
+        SubDirectory => {
+            Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
+            TagTable => 'Image::ExifTool::Canon::FileInfo',
+        },
+    },
     0x10a9 => {
-        Name => 'WhiteBalanceTable',
+        Name => 'ColorBalance',
         Writable => 0,
         SubDirectory => {
             # this offset is necessary because the table contains short rationals
             # (4 bytes long) but the first entry is 2 bytes into the table.
             Start => '2',
-            TagTable => 'Image::ExifTool::Canon::WhiteBalance',
+            TagTable => 'Image::ExifTool::Canon::ColorBalance',
+        },
+    },
+    0x10b5 => { #PH
+        Name => 'RawJpgInfo',
+        SubDirectory => {
+            Validate => 'Image::ExifTool::Canon::Validate($dirData,$subdirStart,$size)',
+            TagTable => 'Image::ExifTool::CanonRaw::RawJpgInfo',
         },
     },
     0x10ae => 'ColorTemperature',
@@ -237,6 +271,10 @@ sub BuildMakerNotes($$$$$$);
     },
     0x1814 => { #3
         Name => 'MeasuredEV',
+        Notes => q{
+            this the Canon name for what should properly be called MeasuredLV, and is
+            offset by about -5 EV from the calculated LV for most models
+        },
         Format => 'float',
     },
     0x1817 => {
@@ -252,20 +290,40 @@ sub BuildMakerNotes($$$$$$);
             TagTable => 'Image::ExifTool::CanonRaw::ExposureInfo',
         },
     },
+    0x1834 => { #PH
+        Name => 'CanonModelID',
+        Writable => 0,
+        PrintHex => 1,
+        Notes => q{
+            this is the complete list of model ID numbers, but note that many of these
+            models do not produce CRW images
+        },
+        PrintConv => \%Image::ExifTool::Canon::canonModelID,
+    },
     0x1835 => {
         Name => 'DecoderTable',
         Writable => 0,
     },
+    0x183b => { #PH
+        # display format for serial number
+        Name => 'SerialNumberFormat',
+        Writable => 'int32u',
+        PrintHex => 1,
+        PrintConv => {
+            0x90000000 => 'Format 1',
+            0xa0000000 => 'Format 2',
+        },
+    },
     0x2005 => {
         Name => 'RawData',
         Writable => 0,
-        ValueConv => '\$val',
+        Binary => 1,
     },
     0x2007 => {
         Name => 'JpgFromRaw',
         Writable => 'resize',  # 'resize' allows this value to change size
         Permanent => 0,
-        ValueConv => '\$val',
+        ValueConv => '$self->ValidateImage(\$val,$tag)',
         ValueConvInv => '$val',
     },
     0x2008 => {
@@ -273,7 +331,7 @@ sub BuildMakerNotes($$$$$$);
         Writable => 'resize',  # 'resize' allows this value to change size
         WriteCheck => '$self->CheckImage(\$val)',
         Permanent => 0,
-        ValueConv => '\$val',
+        ValueConv => '$self->ValidateImage(\$val,$tag)',
         ValueConvInv => '$val',
     },
     # the following entries are subdirectories
@@ -320,6 +378,7 @@ sub BuildMakerNotes($$$$$$);
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
     CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    DATAMEMBER => [ 0, 6 ], # indices of data members to extract when writing
     WRITABLE => 1,
     FORMAT => 'string',
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
@@ -328,15 +387,13 @@ sub BuildMakerNotes($$$$$$);
     0 => {
         Name => 'Make',
         Format => 'string[6]',  # "Canon\0"
-        ValueConv => '$self->{CameraMake} = $val',
-        ValueConvInv => '$val',
+        RawConv => '$self->{CameraMake} = $val',
     },
     6 => {
         Name => 'Model',
         Format => 'string[$size-6]',
         Description => 'Camera Model Name',
-        ValueConv => '$self->{CameraModel} = $val',
-        ValueConvInv => '$val',
+        RawConv => '$self->{CameraModel} = $val',
     },
 );
 
@@ -350,7 +407,8 @@ sub BuildMakerNotes($$$$$$);
     GROUPS => { 0 => 'MakerNotes', 2 => 'Time' },
     0 => {
         Name => 'DateTimeOriginal',
-        Description => 'Shooting Date/Time',
+        Description => 'Date/Time Original',
+        Shift => 'Time',
         ValueConv => 'ConvertUnixTime($val)',
         ValueConvInv => 'GetUnixTime($val)',
         PrintConv => '$self->ConvertDateTime($val)',
@@ -362,7 +420,10 @@ sub BuildMakerNotes($$$$$$);
         ValueConv => '$val / 3600',
         ValueConvInv => '$val * 3600',
     },
-    2 => 'TimeZoneInfo', #3
+    2 => { #3
+        Name => 'TimeZoneInfo',
+        Notes => 'set to 0x80000000 if TimeZoneCode is valid',
+    },
 );
 
 %Image::ExifTool::CanonRaw::ImageFormat = (
@@ -389,6 +450,36 @@ sub BuildMakerNotes($$$$$$);
     },
 );
 
+%Image::ExifTool::CanonRaw::RawJpgInfo = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    FORMAT => 'int16u',
+    FIRST_ENTRY => 1,
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+#    0 => 'RawJpgInfoSize',
+    1 => { #PH
+        Name => 'RawJpgQuality',
+        PrintConv => {
+            1 => 'Economy',
+            2 => 'Normal',
+            3 => 'Fine',
+            5 => 'Superfine',
+        },
+    },
+    2 => { #PH
+        Name => 'RawJpgSize',
+        PrintConv => {
+            0 => 'Large',
+            1 => 'Medium',
+            2 => 'Small',
+        },
+    },
+    3 => 'RawJpgWidth', #PH
+    4 => 'RawJpgHeight', #PH
+);
+
 %Image::ExifTool::CanonRaw::FlashInfo = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
@@ -414,28 +505,15 @@ sub BuildMakerNotes($$$$$$);
     2 => 'AvValue',
 );
 
-%Image::ExifTool::CanonRaw::SensorInfo = (
-    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
-    FORMAT => 'int16s',
-    FIRST_ENTRY => 1,
-    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
-    # Note: Don't make these writable because it confuses Canon decoding software
-    # if these are changed
-    1 => 'SensorWidth',
-    2 => 'SensorHeight',
-    5 => 'SensorLeftBorder', #2
-    6 => 'SensorTopBorder', #2
-    7 => 'SensorRightBorder', #2
-    8 => 'SensorBottomBorder', #2
-);
-
 %Image::ExifTool::CanonRaw::ImageInfo = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
     FORMAT => 'int32u',
     FIRST_ENTRY => 1,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
-    # Note: Don't make these writable because it confuses Canon decoding software
-    # if these are changed
+    # Note: Don't make these writable (except rotation) because it confuses
+    # Canon decoding software if the are changed
     0 => 'ImageWidth', #3
     1 => 'ImageHeight', #3
     2 => { #3
@@ -445,6 +523,7 @@ sub BuildMakerNotes($$$$$$);
     3 => {
         Name => 'Rotation',
         Format => 'int32s',
+        Writable => 'int32s',
     },
     4 => 'ComponentBitDepth', #3
     5 => 'ColorBitDepth', #3
@@ -516,7 +595,7 @@ sub ProcessCanonRaw($$$)
             );
             if ($verbose) {
                 my $fakeInfo = { Name => $name, SubDirectory => { } };
-                $exifTool->VerboseInfo($tag, $fakeInfo,
+                $exifTool->VerboseInfo($tagID, $fakeInfo,
                     'Index'  => $index,
                     'Size'   => $size,
                     'Start'  => $ptr,
@@ -533,10 +612,10 @@ sub ProcessCanonRaw($$$)
             $count = $$tagInfo{Count};
         }
         # get value data
-        my $value;
+        my ($value, $delRawConv);
         if ($valueInDir) {  # is the value data in the directory?
             # this type of tag stores the value in the 'size' and 'ptr' fields
-            $valueDataPos = $dirOffset + $valuePtr;
+            $valueDataPos = $dirOffset + $pt + 4;
             $size = 8;
             $value = substr($buff, $pt+2, $size);
             # set count to 1 by default for normal values in directory
@@ -565,7 +644,8 @@ sub ProcessCanonRaw($$$)
                         }
                     }
                     # force this to be a binary (scalar reference)
-                    $$tagInfo{ValueConv} = '\$val';
+                    $$tagInfo{RawConv} = '\$val';
+                    $delRawConv = 1;
                 }
                 $size = length $value;
                 undef $format;
@@ -581,15 +661,15 @@ sub ProcessCanonRaw($$$)
         if ($verbose) {
             my $val = $value;
             $format and $val = ReadValue(\$val, 0, $format, $count, $size);
-            $exifTool->VerboseInfo($tag, $tagInfo,
-                'Table'  => $rawTagTable,
-                'Index'  => $index,
-                'Value'  => $val,
-                'DataPt' => \$value,
-                'Addr'   => $blockStart + $valueDataPos,
-                'Size'   => $size,
-                'Format' => $format,
-                'Count'  => $count,
+            $exifTool->VerboseInfo($tagID, $tagInfo,
+                Table   => $rawTagTable,
+                Index   => $index,
+                Value   => $val,
+                DataPt  => \$value,
+                DataPos => $valueDataPos,
+                Size    => $size,
+                Format  => $format,
+                Count   => $count,
             );
         }
         if ($buildMakerNotes) {
@@ -619,6 +699,7 @@ sub ProcessCanonRaw($$$)
                 Name     => $name,
                 DataPt   => $dirData,
                 DataLen  => $size,
+                DataPos  => $valueDataPos,
                 DirStart => $subdirStart,
                 DirLen   => $size - $subdirStart,
                 Nesting  => $$dirInfo{Nesting} + 1,
@@ -634,15 +715,9 @@ sub ProcessCanonRaw($$$)
         } else {
             # convert to specified format if necessary
             $format and $value = ReadValue(\$value, 0, $format, $count, $size);
-            my $tagName = $$tagInfo{Name};
-            # check for valid image
-            if (($tagName eq 'JpgFromRaw' or $tagName eq 'ThumbnailImage') and not
-                ($value =~ /^(Binary|\xff\xd8)/ or $exifTool->Options('IgnoreMinorErrors')))
-            {
-                $exifTool->Warn("$tagName is not a valid image", 1);
-            } else {
-                $exifTool->FoundTag($tagInfo, $value);
-            }
+            # save the information
+            $exifTool->FoundTag($tagInfo, $value);
+            delete $$tagInfo{RawConv} if $delRawConv;
         }
     }
     return 1;
@@ -663,7 +738,7 @@ sub ProcessCRW($$)
     SetByteOrder($buff)           or return 0;
     $raf->Read($buff,4) == 4      or return 0;
     $raf->Read($sig,8) == 8       or return 0;  # get file signature
-    $sig eq 'HEAPCCDR'            or return 0;  # validate signature
+    $sig =~ /^HEAP(CCDR|JPGM)/    or return 0;  # validate signature
     my $hlen = Get32u(\$buff, 0);
 
     $raf->Seek(0, 2)              or return 0;  # seek to end of file
@@ -672,7 +747,8 @@ sub ProcessCRW($$)
     # initialize maker note data if building maker notes
     $buildMakerNotes and InitMakerNotes($exifTool);
 
-    $exifTool->SetFileType('Canon RAW');    # set the FileType tag
+    # set the FileType tag unless already done (ie. APP0 CIFF record in JPEG image)
+    $exifTool->SetFileType() unless $exifTool->GetValue('FileType');
 
     # build directory information for main raw directory
     my %dirInfo = (
@@ -693,6 +769,10 @@ sub ProcessCRW($$)
     # finish building maker notes if necessary
     $buildMakerNotes and SaveMakerNotes($exifTool);
 
+    # process trailers if they exist
+    my $trailInfo = Image::ExifTool::IdentifyTrailer($raf);
+    $exifTool->ProcessTrailers($trailInfo) if $trailInfo;
+
     return 1;
 }
 
@@ -702,7 +782,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::CanonRaw - Definitions for Canon CRW file meta information
+Image::ExifTool::CanonRaw - Read Canon RAW (CRW) meta information
 
 =head1 SYNOPSIS
 
@@ -723,10 +803,10 @@ tags.)
 
 =head1 AUTHOR
 
-Copyright 2003-2005, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This library is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =head1 REFERENCES
 
@@ -745,6 +825,7 @@ it under the same terms as Perl itself.
 =head1 SEE ALSO
 
 L<Image::ExifTool::TagNames/CanonRaw Tags>,
+L<Image::ExifTool::Canon(3pm)|Image::ExifTool::Canon>,
 L<Image::ExifTool(3pm)|Image::ExifTool>
 
 =cut
