@@ -1,7 +1,7 @@
 #------------------------------------------------------------------------------
 # File:         Pentax.pm
 #
-# Description:  Definitions for Pentax/Asahi EXIF Maker Notes
+# Description:  Pentax/Asahi EXIF maker notes tags
 #
 # Revisions:    11/25/2003 - P. Harvey Created
 #               02/10/2004 - P. Harvey Completely re-done
@@ -20,6 +20,12 @@
 #               7) Douglas O'Brien private communication (tests with *istD)
 #               8) Denis Bourez private communication
 #               9) Kazumichi Kawabata private communication
+#              10) David Buret private communication (tests with *istD)
+#              11) http://forums.dpreview.com/forums/read.asp?forum=1036&message=17465929
+#              12) Derby Chang private communication
+#              13) http://homepage3.nifty.com/kamisaka/makernote/makernote_pentax.htm
+#              14) Ger Vermeulen private communication (tests with Optio S6)
+#              15) Barney Garrett private communication (tests with Samsung GX-1S)
 #
 # Notes:        See POD documentation at the bottom of this file
 #------------------------------------------------------------------------------
@@ -28,9 +34,9 @@ package Image::ExifTool::Pentax;
 
 use strict;
 use vars qw($VERSION);
-use Image::ExifTool::MakerNotes;
+use Image::ExifTool::Exif;
 
-$VERSION = '1.24';
+$VERSION = '1.40';
 
 # Pentax city codes - PH (from Optio WP)
 my %pentaxCities = (
@@ -104,19 +110,20 @@ my %pentaxCities = (
     67 => 'Nairobi',
     68 => 'Amsterdam',
     69 => 'Stockholm',
+    70 => 'Lisbon', #14
 );
 
 %Image::ExifTool::Pentax::Main = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
     CHECK_PROC => \&Image::ExifTool::Exif::CheckExif,
-    WRITABLE => 1,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    WRITABLE => 1,
     0x0001 => {
         Name => 'PentaxMode',
         Writable => 'int16u',
         PrintConv => {
             0 => 'Auto',
-            1 => 'Night-scene',
+            1 => 'Night Scene',
             2 => 'Manual',
         },
     },
@@ -137,31 +144,90 @@ my %pentaxCities = (
     },
     0x0004 => {
         Name => 'PreviewImageStart',
-        Flags => 'IsOffset',
+        IsOffset => 2,  # code to use original base
         Protected => 2,
         OffsetPair => 0x0003, # point to associated byte count
         DataTag => 'PreviewImage',
         Groups => { 2 => 'Image' },
         Writable => 'int32u',
     },
+    0x0005 => { #13
+        Name => 'PentaxModelID',
+        Writable => 'int32u',
+        PrintHex => 1,
+        PrintConv => { #PH
+            0x0000d => 'Optio 330/430',
+            0x12926 => 'Optio 230',
+            0x12958 => 'Optio 330GS',
+            0x12962 => 'Optio 450/550',
+            0x1296c => 'Optio S',
+            0x12994 => '*ist D',
+            0x129b2 => 'Optio 33L',
+            0x129bc => 'Optio 33LF',
+            0x129c6 => 'Optio 33WR/43WR/555',
+            0x129d5 => 'Optio S4',
+            0x12a02 => 'Optio MX',
+            0x12a0c => 'Optio S40',
+            0x12a16 => 'Optio S4i',
+            0x12a34 => 'Optio 30',
+            0x12a52 => 'Optio S30',
+            0x12a66 => 'Optio 750Z',
+            0x12a70 => 'Optio SV',
+            0x12a75 => 'Optio SVi',
+            0x12a7a => 'Optio X',
+            0x12a8e => 'Optio S5i',
+            0x12a98 => 'Optio S50',
+            0x12aa2 => '*ist DS',
+            0x12ab6 => 'Optio MX4',
+            0x12ac0 => 'Optio S5n',
+            0x12aca => 'Optio WP',
+            0x12afc => 'Optio S55',
+            0x12b10 => 'Optio S5z',
+            0x12b1a => '*ist DL',
+            0x12b24 => 'Optio S60',
+            0x12b2e => 'Optio S45',
+            0x12b38 => 'Optio S6',
+            0x12b4c => 'Optio WPi', #13
+            0x12b56 => 'BenQ DC X600',
+            0x12b60 => '*ist DS2',
+            0x12b62 => 'Samsung GX-1S',
+            0x12b6a => 'Optio A10',
+            0x12b7e => '*ist DL2',
+            0x12b80 => 'Samsung GX-1L',
+            0x12b9c => 'K100D',
+            0x12b9d => 'K110D',
+            0x12bb0 => 'Optio T10',
+            0x12be2 => 'Optio W10',
+            0x12bf6 => 'Optio M10',
+            0x12c1e => 'K10D',
+            0x12c20 => 'Samsung GX10',
+            0x12c28 => 'Optio S7',
+            0x12c32 => 'Optio M20',
+            0x12c3c => 'Optio W20',
+            0x12c46 => 'Optio A20',
+        },
+    },
     0x0006 => { #5
         # Note: Year is int16u in MM byte ordering regardless of EXIF byte order
         Name => 'Date',
+        Groups => { 2 => 'Time' },
         Writable => 'undef',
         Count => 4,
+        Shift => 'Time',
         ValueConv => 'length($val)==4 ? sprintf("%.4d:%.2d:%.2d",unpack("nC2",$val)) : "Unknown ($val)"',
         ValueConvInv => 'my @v=split /:/, $val;pack("nC2",$v[0],$v[1],$v[2])',
     },
     0x0007 => { #5
         Name => 'Time',
+        Groups => { 2 => 'Time' },
         Writable => 'undef',
         Count => 3,
+        Shift => 'Time',
         ValueConv => 'length($val)>=3 ? sprintf("%.2d:%.2d:%.2d",unpack("C3",$val)) : "Unknown ($val)"',
         ValueConvInv => 'pack("C3",split(/:/,$val))',
     },
     0x0008 => {
         Name => 'Quality',
-        Description => 'Image Quality',
         Writable => 'int16u',
         PrintConv => {
             0 => 'Good',
@@ -182,15 +248,23 @@ my %pentaxCities = (
             3 => '1280x960', #PH (Optio WP)
             4 => '1600x1200',
             5 => '2048x1536',
-            8 => '2560x1920', #PH (Optio WP)
+            8 => '2560x1920 or 2304x1728', #PH (Optio WP) or #14
+            10 => '3264x2448', #13
             19 => '320x240', #PH (Optio WP)
+            20 => '2288x1712', #13
             21 => '2592x1944',
-            22 => '2304x1728', #2
+            22 => '2304x1728 or 2592x1944', #2 or #14
+            23 => '3056x2296', #13
+            25 => '2816x2212 or 2816x2112', #13 or #14
+            '0 0' => '2304x1728', #13
+            '5 0' => '2048x1536', #13
+            '8 0' => '2560x1920', #13
             '32 2' => '960x640', #7
             '33 2' => '1152x768', #7
             '34 2' => '1536x1024', #7
             '35 1' => '2400x1600', #7
             '36 0' => '3008x2008',  #PH
+            '37 0' => '3008x2000', #13
         },
     },
     # 0x000a - (See note below)
@@ -199,19 +273,23 @@ my %pentaxCities = (
         Writable => 'int16u',
         PrintConv => {
             0 => 'Program', #PH
+            2 => 'Program AE', #13
+            3 => 'Manual', #13
             5 => 'Portrait',
             6 => 'Landscape',
             8 => 'Sport', #PH
             9 => 'Night Scene',
             11 => 'Soft', #PH
             12 => 'Surf & Snow',
-            13 => 'Sunset',
+            13 => 'Sunset or Candlelight', #14 (Candlelight)
             14 => 'Autumn',
-            15 => 'Flower',
+            15 => 'Macro',
             17 => 'Fireworks',
             18 => 'Text',
             19 => 'Panorama', #PH
             30 => 'Self Portrait', #PH
+            31 => 'Illustrations', #13
+            33 => 'Digital Filter', #13
             37 => 'Museum', #PH
             38 => 'Food', #PH
             40 => 'Green Mode', #PH
@@ -222,6 +300,15 @@ my %pentaxCities = (
             54 => 'Candlelight', #PH
             55 => 'Natural Skin Tone', #PH
             56 => 'Synchro Sound Record', #PH
+            58 => 'Frame Composite', #14
+            '0 0' => 'Auto', #13
+            '0 2' => 'Program AE', #13
+            '5 2' => 'Portrait', #13
+            '6 2' => 'Landscape', #13
+            '9 1' => 'Night Scene', #13
+            '13 1' => 'Candlelight', #13
+            '15 1' => 'Macro', #13
+            '255 0' => 'Digital Filter?', #13
         },
     },
     0x000c => { #PH
@@ -265,12 +352,32 @@ my %pentaxCities = (
             },
         },
     ],
+    0x000e => { #7
+        Name => 'AFPointSelected',
+        Writable => 'int16u',
+        PrintConv => {
+            0xffff => 'Auto',
+            0xfffe => 'Fixed Center',
+            1 => 'Upper-left',
+            2 => 'Top',
+            3 => 'Upper-right',
+            4 => 'Left',
+            5 => 'Mid-left',
+            6 => 'Center',
+            7 => 'Mid-right',
+            8 => 'Right',
+            9 => 'Lower-left',
+            10 => 'Bottom',
+            11 => 'Lower-right',
+        },
+    },
     0x000f => { #PH
         Name => 'AutoAFPoint',
         Writable => 'int16u',
         PrintHex => 1,
         PrintConv => {
             0xffff => 'None',
+            0 => 'Multiple', #14
             1 => 'Top-left',
             2 => 'Top-center',
             3 => 'Top-right',
@@ -285,11 +392,10 @@ my %pentaxCities = (
     0x0010 => { #PH
         Name => 'FocusPosition',
         Writable => 'int16u',
-        Notes => 'Related to focus distance but effected by focal length',
+        Notes => 'related to focus distance but affected by focal length',
     },
     0x0012 => { #PH
         Name => 'ExposureTime',
-        Description => 'Shutter Speed',
         Writable => 'int32u',
         Priority => 0,
         ValueConv => '$val * 1e-5',
@@ -299,10 +405,9 @@ my %pentaxCities = (
     },
     0x0013 => { #PH
         Name => 'FNumber',
-        Description => 'Aperture',
         Writable => 'int16u',
         Priority => 0,
-        ValueConv => '$val * 0.1',
+        ValueConv => '$val / 10',
         ValueConvInv => '$val * 10',
         PrintConv => 'sprintf("%.1f",$val)',
         PrintConvInv => '$val',
@@ -310,7 +415,6 @@ my %pentaxCities = (
     # ISO Tag - Entries confirmed by W. Smith 12 FEB 04
     0x0014 => {
         Name => 'ISO',
-        Description => 'ISO Speed',
         Writable => 'int16u',
         Priority => 0,
         PrintConv => {
@@ -344,13 +448,12 @@ my %pentaxCities = (
     0x0016 => { #PH
         Name => 'ExposureCompensation',
         Writable => 'int16u',
-        ValueConv => '($val - 50) * 0.1',
+        ValueConv => '($val - 50) / 10',
         ValueConvInv => 'int($val * 10 + 50.5)',
         PrintConv => 'Image::ExifTool::Exif::ConvertFraction($val)',
         PrintConvInv => 'eval $val',
     },
-    # AE Metering Mode Tag - W. Smith 12 FEB 04
-    0x0017 => {
+    0x0017 => { #3
         Name => 'MeteringMode',
         Writable => 'int16u',
         PrintConv => {
@@ -359,8 +462,16 @@ my %pentaxCities = (
             2 => 'Spot',
         },
     },
-    # White Balance Tag - W. Smith 12 FEB 04
-    0x0019 => {
+    0x0018 => { #7/PH
+        Name => 'ExposureBracketStep',
+        Writable => 'int16',
+        # don't apply conversion if this is 2 integers
+        ValueConv => '$val=~/ / ? $val : $val / 3',
+        ValueConvInv => '$val=~/ / ? $val : int($val * 3 + 0.5)',
+        PrintConv => '$val=~/ / ? $val : sprintf("%.1f",$val)',
+        PrintConvInv => '$val',
+    },
+    0x0019 => { #3
         Name => 'WhiteBalance',
         Writable => 'int16u',
         PrintConv => {
@@ -370,16 +481,23 @@ my %pentaxCities = (
             3 => 'Fluorescent', #2
             4 => 'Tungsten',
             5 => 'Manual',
+            6 => 'DaylightFluorescent', #13
+            7 => 'DaywhiteFluorescent', #13
+            8 => 'WhiteFluorescent', #13
+            9 => 'Flash', #13
+            10 => 'Cloudy', #13
+            65534 => 'Unknown', #13
+            65535 => 'User Selected', #13
         },
     },
     0x001a => { #5
         Name => 'WhiteBalanceMode',
         Writable => 'int16u',
         PrintConv => {
-            1 => 'Auto (Daylight?)',
-            2 => 'Auto (Shade?)',
-            3 => 'Auto (Flash?)',
-            4 => 'Auto (Tungsten?)',
+            1 => 'Auto (Daylight)',
+            2 => 'Auto (Shade)',
+            3 => 'Auto (Flash)',
+            4 => 'Auto (Tungsten)',
             0xffff => 'User-Selected',
             0xfffe => 'Preset (Fireworks?)', #PH
         },
@@ -396,35 +514,41 @@ my %pentaxCities = (
         ValueConv => '$val / 256',
         ValueConvInv => 'int($val * 256 + 0.5)',
     },
-    # Would be nice if there was a general way to determine units for FocalLength.
-    # Optio 550 uses .1mm while *istD, Optio S and Optio WP use .01 - PH
     0x001d => [
+        # Would be nice if there was a general way to determine units for FocalLength...
         {
-            Condition => '$self->{CameraModel} =~ /(\*ist D|Optio [A-Z])/',
+            # Optio 30, 33WR, 43WR, 450, 550, 555, 750Z, X
             Name => 'FocalLength',
+            Condition => '$self->{CameraModel} =~ /^PENTAX Optio (30|33WR|43WR|450|550|555|750Z|X)\b/',
             Writable => 'int32u',
-            ValueConv => '$val * 0.01',
-            ValueConvInv => '$val / 0.01',
+            Priority => 0,
+            ValueConv => '$val / 10',
+            ValueConvInv => '$val * 10',
             PrintConv => 'sprintf("%.1fmm",$val)',
             PrintConvInv => '$val=~s/mm//;$val',
         },
         {
+            # K100D, Optio 230, 330GS, 33L, 33LF, A10, M10, MX, MX4, S, S30,
+            # S4, S4i, S5i, S5n, S5z, S6, S45, S50, S55, S60, SV, Svi, W10, WP,
+            # *ist D, DL, DL2, DS, DS2
+            # (Note: the Optio S6 seems to report the minimum focal length - PH)
             Name => 'FocalLength',
             Writable => 'int32u',
-            ValueConv => '$val * 0.1',
-            ValueConvInv => '$val / 0.1',
+            Priority => 0,
+            ValueConv => '$val / 100',
+            ValueConvInv => '$val * 100',
             PrintConv => 'sprintf("%.1fmm",$val)',
             PrintConvInv => '$val=~s/mm//;$val',
         },
     ],
-    # Digital Zoom Tag - W. Smith 12 FEB 04
-    0x001e => {
+    0x001e => { #3
         Name => 'DigitalZoom',
         Writable => 'int16u',
+        ValueConv => '$val / 100', #14
+        ValueConvInv => '$val * 100', #14
     },
     0x001f => {
         Name => 'Saturation',
-        Notes => 'Pentax models',
         Writable => 'int16u',
         PrintConv => {
             0 => 'Low', #PH
@@ -440,7 +564,6 @@ my %pentaxCities = (
     },
     0x0020 => {
         Name => 'Contrast',
-        Notes => 'Pentax models',
         Writable => 'int16u',
         PrintConv => {
             0 => 'Low', #PH
@@ -456,7 +579,6 @@ my %pentaxCities = (
     },
     0x0021 => {
         Name => 'Sharpness',
-        Notes => 'Pentax models',
         Writable => 'int16u',
         PrintConv => {
             0 => 'Soft', #PH
@@ -504,14 +626,115 @@ my %pentaxCities = (
         Name => 'FrameNumber',
         Writable => 'int32u',
     },
+    # 0x002b - definitely exposure related somehow (PH)
     # 0x0032 - normally 4 zero bytes, but "\x02\0\0\0" for a cropped pic (PH)
+    # and "\0\0\0\x04" for Digital filter (ref 13)
+    # and "\x04\0\0\0" for Color filter (ref 13)
+    0x0033 => { #PH (K110D/K100D)
+        Name => 'PictureMode',
+        Writable => 'int8u',
+        Count => 3,
+        PrintConv => {
+            # Program dial modes
+            '0 0 0'  => 'Program', # K110D
+            '0 4 0'  => 'Standard', #13
+            '0 5 0'  => 'Portrait', # K110D
+            '0 6 0'  => 'Landscape', # K110D
+            '0 7 0'  => 'Macro', # K110D
+            '0 8 0'  => 'Sport', # K110D
+            '0 9 0'  => 'Night Scene Portrait', # K110D
+            '0 10 0' => 'No Flash', # K110D
+            # SCN modes (menu-selected)
+            '0 11 0' => 'Night Scene', # K100D
+            '0 12 0' => 'Surf & Snow', # K100D
+            '0 13 0' => 'Text', # K100D
+            '0 14 0' => 'Sunset', # K100D
+            '0 15 0' => 'Kids', # K100D
+            '0 16 0' => 'Pet', # K100D
+            '0 17 0' => 'Candlelight', # K100D
+            '0 18 0' => 'Museum', # K100D
+            # AUTO PICT modes (auto-selected)
+            '1 4 0'  => 'Auto PICT (Standard)', #13
+            '1 5 0'  => 'Auto PICT (Portrait)', #7 (K100D)
+            '1 6 0'  => 'Auto PICT (Landscape)', # K110D
+            '1 7 0'  => 'Auto PICT (Macro)', #13
+            '1 8 0'  => 'Auto PICT (Sport)', #13
+            # Manual dial modes
+            '2 0 0'  => 'Program AE', #13
+            '3 0 0'  => 'Green Mode', # (unconfirmed)
+            '4 0 0'  => 'Shutter Speed Priority', # K110D
+            '5 0 0'  => 'Aperture Priority', # K110D
+            '8 0 0'  => 'Manual', # K110D
+            '9 0 0'  => 'Bulb', # K110D
+            # *istD modes (ref 7)
+            '2 0 1'  => 'Program AE', #7
+            '2 1 1'  => 'Hi-speed Program', #7
+            '2 2 1'  => 'DOF Program', #7
+            '2 3 1'  => 'MTF Program', #7
+            '3 0 1'  => 'Green Mode', #7
+            '4 0 1'  => 'Shutter Speed Priority', #7
+            '5 0 1'  => 'Aperture Priority', #7
+            '6 0 1'  => 'Program Tv Shift', #7
+            '7 0 1'  => 'Program Av Shift', #7
+            '8 0 1'  => 'Manual', #7
+            '9 0 1'  => 'Bulb', #7
+            '10 0 1' => 'Aperture Priority (Off-Auto-Aperture)', #7
+            '11 0 1' => 'Manual (Off-Auto-Aperture)', #7
+            '12 0 1' => 'Bulb (Off-Auto-Aperture)', #7
+        },
+    },
+    0x0034 => { #7/PH
+        Name => 'DriveMode',
+        Writable => 'int8u',
+        Count => 4,
+        PrintConv => {
+            '0 0 0 0' => 'Single-frame',
+            '1 0 0 0' => 'Continuous',
+            '0 1 0 0' => 'Self-timer (12 sec)',
+            '0 2 0 0' => 'Self-timer (2 sec)',
+            '0 0 1 0' => 'Remote Control?', # needs validation
+            '0 0 0 1' => 'Multiple Exposure',
+        },
+    },
+    0x0037 => { #13
+        Name => 'ColorSpace',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'sRGB',
+            1 => 'Adobe RGB',
+        },
+    },
     0x0039 => { #PH
         Name => 'RawImageSize',
         Writable => 'int16u',
         Count => 2,
         PrintConv => '$_=$val;s/ /x/;$_',
     },
-    0x003f => {     #PH
+    0x003c => { #7/PH
+        Name => 'AFPointsUsed',
+        # not writable because I'm not decoding these 4 bytes fully:
+        # Nibble pattern: XSSSYUUU
+        # X = unknown (AF focused flag?, 0 or 1)
+        # SSS = selected AF point bitmask (0x000 or 0x7ff if unused)
+        # Y = unknown (observed 0,6,7,b,e, always 0 if SSS is 0x000 or 0x7ff)
+        # UUU = af points used
+        Format => 'int32u',
+        ValueConv => '$val & 0x7ff', # ignore other bits for now
+        PrintConv => { BITMASK => {
+            0 => 'Upper-left',
+            1 => 'Top',
+            2 => 'Upper-right',
+            3 => 'Left',
+            4 => 'Mid-left',
+            5 => 'Center',
+            6 => 'Mid-right',
+            7 => 'Right',
+            8 => 'Lower-left',
+            9 => 'Bottom',
+            10 => 'Lower-right',
+        } },
+    },
+    0x003f => { #PH
         Name => 'LensType',
         Writable => 'int8u',
         Count => 2,
@@ -527,13 +750,16 @@ my %pentaxCities = (
             '3 22' => 'smc PENTAX-F FISH-EYE 17-28mm F3.5-4.5',
             '3 23' => 'smc PENTAX-F 100-300mm F4.5-5.6',
             '3 24' => 'smc PENTAX-F 35-135mm F3.5-4.5',
-            '3 25' => 'smc PENTAX-F 35-105mm F4-5.6',
+            '3 25' => 'smc PENTAX-F 35-105mm F4-5.6 or SIGMA AF 28-300 F3.5-5.6 DL IF', #11 (sigma)
+            # or '3 25' => 'Tokina 80-200mm F2.8 ATX-Pro', #12
             '3 26' => 'smc PENTAX-F* 250-600mm F5.6 ED[IF]',
             '3 27' => 'smc PENTAX-F 28-80mm F3.5-4.5',
             '3 28' => 'smc PENTAX-F 35-70mm F3.5-4.5',
-            '3 29' => 'PENTAX-F 28-80mm F3.5-4.5',
+            # or '3 28' => 'Tokina 19-35mm F3.5-4.5 AF', #12
+            '3 29' => 'PENTAX-F 28-80mm F3.5-4.5 or SIGMA AF 18-125mm F3.5-5.6 DC', #11 (sigma)
             '3 30' => 'PENTAX-F 70-200mm F4-5.6',
             '3 31' => 'smc PENTAX-F 70-210mm F4-5.6',
+            # or '3 31' => 'Tokina AF 730 75-300mm F4.5-5.6',
             '3 32' => 'smc PENTAX-F 50mm F1.4',
             '3 33' => 'smc PENTAX-F 50mm F1.7',
             '3 34' => 'smc PENTAX-F 135mm F2.8 [IF]',
@@ -543,28 +769,43 @@ my %pentaxCities = (
             '3 39' => 'smc PENTAX-F* 600mm F4 ED[IF]',
             '3 40' => 'smc PENTAX-F MACRO 100mm F2.8',
             '3 41' => 'smc PENTAX-F MACRO 50mm F2.8',
-            '3 44' => 'SIGMA 18-50mm F3.5-5.6 DC',
+            '3 44' => 'SIGMA 18-50mm F3.5-5.6 DC, 12-24mm F4.5 EX DG or Tamron 35-90mm F4 AF', #4,12,12
             '3 46' => 'SIGMA APO 70-200mm F2.8 EX',
             '3 50' => 'smc PENTAX-FA 28-70mm F4 AL',
             '3 51' => 'SIGMA 28mm F1.8 EX DG ASPHERICAL MACRO',
             '3 52' => 'smc PENTAX-FA 28-200mm F3.8-5.6 AL[IF]',
             '3 53' => 'smc PENTAX-FA 28-80mm F3.5-5.6 AL',
+            '3 247' => 'smc PENTAX-DA FISH-EYE 10-17mm F3.5-4.5 ED[IF]',
+            '3 248' => 'smc PENTAX-DA 12-24mm F4 ED AL[IF]',
+            '3 250' => 'smc PENTAX-DA 50-200mm F4-5.6 ED',
+            '3 251' => 'smc PENTAX-DA 40mm F2.8 Limited',
+            '3 252' => 'smc PENTAX-DA 18-55mm F3.5-5.6 AL',
             '3 253' => 'smc PENTAX-DA 14mm F2.8 ED[IF]',
             '3 254' => 'smc PENTAX-DA 16-45mm F4 ED AL',
-            '3 255' => 'SIGMA 18-200mm F3.5-6.3 DC', #8
+            '3 255' => 'SIGMA',
+            # '3 255' => 'SIGMA 18-200mm F3.5-6.3 DC', #8
+            # '3 255' => 'SIGMA DL-II 35-80mm F4-5.6', #12
+            # '3 255' => 'SIGMA DL Zoom 75-300mm F4-5.6', #12
+            # '3 255' => 'SIGMA DF EX Aspherical 28-70mm F2.8', #12
             '4 1' => 'smc PENTAX-FA SOFT 28mm F2.8',
             '4 2' => 'smc PENTAX-FA 80-320mm F4.5-5.6',
             '4 3' => 'smc PENTAX-FA 43mm F1.9 Limited',
             '4 6' => 'smc PENTAX-FA 35-80mm F4-5.6',
             '4 15' => 'smc PENTAX-FA 28-105mm F4-5.6 [IF]',
-            '4 19' => 'TAMRON SP AF 90mm F2.8',
+            '4 16' => 'TAMRON AF 80-210mm F4-5.6 (178D)', #13
+            '4 19' => 'TAMRON SP AF 90mm F2.8 (172E)',
             '4 20' => 'smc PENTAX-FA 28-80mm F3.5-5.6',
+            '4 22' => 'TOKINA 28-80mm F3.5-5.6', #13
             '4 23' => 'smc PENTAX-FA 20-35mm F4 AL',
             '4 24' => 'smc PENTAX-FA 77mm F1.8 Limited',
+            '4 25' => 'TAMRON SP AF 14mm F2.8', #13
             '4 26' => 'smc PENTAX-FA MACRO 100mm F3.5',
+            '4 27' => 'TAMRON AF28-300mm F/3.5-6.3 LD Aspherical[IF]Macro (285D)',
             '4 28' => 'smc PENTAX-FA 35mm F2 AL',
             '4 34' => 'smc PENTAX-FA 24-90mm F3.5-4.5 AL[IF]',
             '4 35' => 'smc PENTAX-FA 100-300mm F4.7-5.8',
+            '4 36' => 'TAMRON AF70-300mm F/4-5.6 LD MACRO (572D)',
+            '4 37' => 'TAMRON SP AF 24-135mm F3.5-5.6 AD AL (190D)', #13
             '4 38' => 'smc PENTAX-FA 28-105mm F3.2-4.5 AL[IF]',
             '4 39' => 'smc PENTAX-FA 31mm F1.8AL Limited',
             '4 41' => 'TAMRON AF 28-200mm Super Zoom F3.8-5.6 Aspherical XR [IF] MACRO (A03)',
@@ -574,6 +815,14 @@ my %pentaxCities = (
             '4 46' => 'smc PENTAX-FA J 28-80mm F3.5-5.6 AL',
             '4 47' => 'smc PENTAX-FA J 18-35mm F4-5.6 AL',
             '4 49' => 'TAMRON SP AF 28-75mm F2.8 XR Di (A09)',
+            '4 51' => 'smc PENTAX-D FA 50mmF2.8 MACRO',
+            '4 52' => 'smc PENTAX-D FA 100mmF2.8 MACRO',
+            '4 244' => 'smc PENTAX-DA 21mm F3.2 AL Limited', #9
+            '4 245' => 'Schneider D-XENON 50-200mm', #15
+            '4 246' => 'Schneider D-XENON 18-55mm', #15
+            '4 247' => 'smc PENTAX-DA 10-17mm F3.5-4.5 ED [IF] Fisheye zoom', #10
+            '4 248' => 'smc PENTAX-DA 12-24mm F4 ED AL [IF]', #10
+            '4 249' => 'TAMRON XR DiII 18-200mm F3.5-6.3 (A14)',
             '4 250' => 'smc PENTAX-DA 50-200mm F4-5.6 ED', #8
             '4 251' => 'smc PENTAX-DA 40mm F2.8 Limited', #9
             '4 252' => 'smc PENTAX-DA 18-55mm F3.5-5.6 AL', #8
@@ -607,19 +856,30 @@ my %pentaxCities = (
             '6 10' => 'smc PENTAX-FA* 400mm F5.6 ED[IF]',
             '6 13' => 'smc PENTAX-FA* 400mm F5.6 ED[IF]',
             '6 14' => 'smc PENTAX-FA* MACRO 200mm F4 ED[IF]',
+            '7 243' => 'smc PENTAX-DA 70mm F2.4 Limited', #PH (K10D)
+            '7 244' => 'smc PENTAX-DA 16-45mm F4 ED AL', #PH (K10D)
         },
     },
     # 0x0041 - increments for each cropped pic (PH)
-    0x0200 => {
-        Name => 'BlackPoint', #5
+    # 0x0047 - increments every few pictures for unknown reason (PH)
+    0x0049 => { #13
+        Name => 'NoiseReduction',
+        Writable => 'int16u',
+        PrintConv => { 0 => 'Off', 1 => 'On' },
+    },
+    # 0x004f - PictureFinish ? (ref 13)
+    0x0200 => { #5
+        Name => 'BlackPoint',
         Writable => 'int16u',
         Count => 4,
     },
-    0x0201 => {
-        Name => 'WhitePoint', #5
+    0x0201 => { #5
+        Name => 'WhitePoint',
         Writable => 'int16u',
         Count => 4,
     },
+    # 0x0205 - Also stores PictureMode (PH)
+    # 0x0207 - LensInformation ? (ref 13) - includes focus distance!
     0x03fe => { #PH
         Name => 'DataDump',
         Writable => 0,
@@ -641,16 +901,20 @@ my %pentaxCities = (
             TagTable => 'Image::ExifTool::PrintIM::Main',
         },
     },
-    0x1000 => {
-        Name => 'HometownCityCode', #PH
+    0x1000 => { #PH
+        Name => 'HometownCityCode',
         Writable => 'undef',
         Count => 4,
     },
-    0x1001 => {
-        Name => 'DestinationCityCode', #PH
+    0x1001 => { #PH
+        Name => 'DestinationCityCode',
         Writable => 'undef',
         Count => 4,
     },
+    0x2000 => { #PH (Optio 330RS)
+        Name => 'PreviewImageData',
+        Binary => 1,
+    },
 );
 
 # NOTE: These are from Image::MakerNotes::Pentax.pm, but they don't seem to work - PH
@@ -680,29 +944,33 @@ my %pentaxCities = (
 #        },
 #    },
 
-# tags in Pentax MOV videos (PH - tests with Optio WP)
+# tags in Pentax QuickTime videos (PH - tests with Optio WP)
+# (note: very similar to information in Nikon videos)
 %Image::ExifTool::Pentax::MOV = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
-    GROUPS => { 2 => 'Camera' },
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     NOTES => 'This information is found in Pentax MOV video images.',
+    0x00 => {
+        Name => 'Make',
+        Format => 'string[6]',
+        PrintConv => 'ucfirst(lc($val))',
+    },
     0x26 => {
         Name => 'ExposureTime',
-        Description => 'Shutter Speed',
         Format => 'int32u',
         ValueConv => '$val ? 10 / $val : 0',
         PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
     },
     0x2a => {
         Name => 'FNumber',
-        Description => 'Aperture',
         Format => 'int32u',
-        ValueConv => '$val * 0.1',
+        ValueConv => '$val / 10',
         PrintConv => 'sprintf("%.1f",$val)',
     },
     0x32 => {
         Name => 'ExposureCompensation',
         Format => 'int32s',
-        ValueConv => '$val * 0.1',
+        ValueConv => '$val / 10',
         PrintConv => 'Image::ExifTool::Exif::ConvertFraction($val)',
     },
     0x44 => {
@@ -720,12 +988,11 @@ my %pentaxCities = (
     0x48 => {
         Name => 'FocalLength',
         Writable => 'int32u',
-        ValueConv => '$val * 0.1',
+        ValueConv => '$val / 10',
         PrintConv => 'sprintf("%.1fmm",$val)',
     },
     0xaf => {
         Name => 'ISO',
-        Description => 'ISO Speed',
         Format => 'int16u',
     },
 );
@@ -737,7 +1004,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::Pentax - Definitions for Pentax/Asahi maker notes
+Image::ExifTool::Pentax - Pentax/Asahi maker notes tags
 
 =head1 SYNOPSIS
 
@@ -763,7 +1030,7 @@ offset in the middle of nowhere, and the Optio 550 uses different (and
 totally illogical) bases for different menu entries.  Very weird.  (It
 wouldn't surprise me if Pentax can't read their own maker notes!)  Luckily,
 there are only a few entries in the maker notes which are large enough to
-require offsets, so this doesn't effect much useful information.  ExifTool
+require offsets, so this doesn't affect much useful information.  ExifTool
 attempts to make sense of this fiasco by making an assumption about where
 the information should be stored to deduce the correct offsets.
 
@@ -786,15 +1053,15 @@ the information should be stored to deduce the correct offsets.
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to Wayne Smith, John Francis and Douglas O'Brien for help figuring
-out some Pentax tags, and to Denis Bourez and Kazumichi Kawabata for adding
-to the LensType list.
+out some Pentax tags, and to Denis Bourez, Kazumichi Kawabata, David Buret
+and Barney Garrett for adding to the LensType list.
 
 =head1 AUTHOR
 
-Copyright 2003-2005, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This library is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
