@@ -1,7 +1,7 @@
 #------------------------------------------------------------------------------
 # File:         WriteXMP.pl
 #
-# Description:  Routines for writing XMP metadata
+# Description:  Write XMP meta information
 #
 # Revisions:    12/19/2004 - P. Harvey Created
 #
@@ -28,6 +28,7 @@
 package Image::ExifTool::XMP;
 
 use strict;
+use Image::ExifTool qw(:DataAccess :Utils);
 
 sub CheckXMP($$$);
 sub SetPropertyPath($$;$$);
@@ -89,6 +90,33 @@ my %xmpStruct = (
         h           => { },
         unit        => { },
     },
+    Colorant => {
+        NAMESPACE => 'xapG',
+        swatchName  => { },
+        mode        => { },
+        type        => { },
+        cyan        => { },
+        magenta     => { },
+        yellow      => { },
+        black       => { },
+        red         => { },
+        green       => { },
+        blue        => { },
+        L           => { },
+        A           => { },
+        B           => { },
+    },
+    Font => {
+        NAMESPACE => 'stFnt',
+        fontName    => { },
+        fontFamily  => { },
+        fontFace    => { },
+        fontType    => { },
+        versionString => { },
+        composite   => { },
+        fontFileName=> { },
+        childFontFiles=> { List => 'Seq' },
+    },
     # the following stuctures are different:  They don't have
     # their own namespaces -- instead they use the parent namespace
     Flash => {
@@ -130,6 +158,57 @@ my %xmpStruct = (
         CiTelWork   => { },
         CiUrlWork   => { },
     },
+    # Dynamic Media structures
+    BeatSpliceStretch => {
+        NAMESPACE => 'xmpDM',
+        useFileBeatsMarker  => { },
+        riseInDecibel       => { },
+        riseInTimeDuration  => { },
+    },
+    Marker => {
+        NAMESPACE => 'xmpDM',
+        startTime   => { },
+        duration    => { },
+        comment     => { },
+        name        => { },
+        location    => { },
+        target      => { },
+        type        => { },
+    },
+    Media => {
+        NAMESPACE => 'xmpDM',
+        path        => { },
+        track       => { },
+        startTime   => { },
+        duration    => { },
+        managed     => { },
+        webStatement=> { },
+    },
+    ProjectLink => {
+        NAMESPACE => 'xmpDM',
+        type        => { },
+        path        => { },
+    },
+    ResampleStretch => {
+        NAMESPACE => 'xmpDM',
+        quality     => { },
+    },
+    Time => {
+        NAMESPACE => 'xmpDM',
+        value       => { },
+        scale       => { },
+    },
+    Timecode => {
+        NAMESPACE => 'xmpDM',
+        timeValue   => { },
+        timeFormat  => { },
+    },
+    TimeScaleStretch => {
+        NAMESPACE => 'xmpDM',
+        quality     => { },
+        frameSize   => { },
+        frameOverlappingPercentage => { },
+    },
 );
 
 # Lookup to translate our namespace prefixes into URI's.  This list need
@@ -146,73 +225,59 @@ my %nsURI = (
     pdfx      => 'http://ns.adobe.com/pdfx/1.3/',
     photoshop => 'http://ns.adobe.com/photoshop/1.0/',
     rdf       => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
+    rdfs      => 'http://www.w3.org/2000/01/rdf-schema#',
     stDim     => 'http://ns.adobe.com/xap/1.0/sType/Dimensions#',
     stEvt     => 'http://ns.adobe.com/xap/1.0/sType/ResourceEvent#',
+    stFnt     => 'http://ns.adobe.com/xap/1.0/sType/Font#',
     stJob     => 'http://ns.adobe.com/xap/1.0/sType/Job#',
     stRef     => 'http://ns.adobe.com/xap/1.0/sType/ResourceRef#',
     stVer     => 'http://ns.adobe.com/xap/1.0/sType/Version#',
     tiff      => 'http://ns.adobe.com/tiff/1.0/',
    'x'        => 'adobe:ns:meta/',
-    xapGImg   => 'http://ns/adobe.com/xap/1.0/g/img/',
+    xapG      => 'http://ns.adobe.com/xap/1.0/g/',
+    xapGImg   => 'http://ns.adobe.com/xap/1.0/g/img/',
     xmp       => 'http://ns.adobe.com/xap/1.0/',
     xmpBJ     => 'http://ns.adobe.com/xap/1.0/bj/',
+    xmpDM     => 'http://ns.adobe.com/xmp/1.0/DynamicMedia/',
     xmpMM     => 'http://ns.adobe.com/xap/1.0/mm/',
     xmpRights => 'http://ns.adobe.com/xap/1.0/rights/',
     xmpTPg    => 'http://ns.adobe.com/xap/1.0/t/pg/',
-    xmpidq    => 'http://ns/adobe.com/xmp/Identifier/qual/1.0',
+    xmpidq    => 'http://ns.adobe.com/xmp/Identifier/qual/1.0/',
     Iptc4xmpCore => 'http://iptc.org/std/Iptc4xmpCore/1.0/xmlns/',
     xmpPLUS   => 'http://ns.adobe.com/xap/1.0/PLUS/',
     dex       => 'http://ns.optimasc.com/dex/1.0/',
 );
 
-# these are the attributes that we handle for properties that contain
-# sub-properties.  Attributes for simple properties are easy, and we
-# just copy them over.  These are harder since we don't store attributes
-# for properties without simple values.  (maybe this will change...)
-my %recognizedAttrs = (
-    'x:xaptk' => 1,
-    'x:xmptk' => 1,
-    'about' => 1,
-    'rdf:about' => 1,
-    'rdf:parseType' => 1,
-);
-
-my $seeded;     # flag indicating we seeded our random number generateor
 my $x_toolkit = "x:xmptk='Image::ExifTool $Image::ExifTool::VERSION'";
 my $rdfDesc = 'rdf:Description';
 #
 # packet/xmp/rdf headers and trailers
 #
 my $pktOpen = "<?xpacket begin='\xef\xbb\xbf' id='W5M0MpCehiHzreSzNTczkc9d'?>\n";
+my $xmlOpen = "\xef\xbb\xbf<?xml version='1.0' encoding='UTF-8'?>\n";
 my $xmpOpen = "<x:xmpmeta xmlns:x='$nsURI{x}' $x_toolkit>\n";
 my $rdfOpen = "<rdf:RDF xmlns:rdf='$nsURI{rdf}'>\n";
 my $rdfClose = "</rdf:RDF>\n";
 my $xmpClose = "</x:xmpmeta>\n";
-my $pktClose =  "<?xpacket end='w'?>";
+my $pktCloseW =  "<?xpacket end='w'?>"; # writable by default
+my $pktCloseR =  "<?xpacket end='r'?>";
 
 # Update XMP tag tables when this library is loaded:
 # - generate all TagID's (required when writing)
 # - generate PropertyPath for structure elements
 # - add necessary inverse conversion routines
 {
-    my $mainTable = Image::ExifTool::GetTagTable('Image::ExifTool::XMP::Main');
-    Image::ExifTool::GenerateTagIDs($mainTable);
-    my $mainTag;
+    my $mainTable = GetTagTable('Image::ExifTool::XMP::Main');
+    GenerateTagIDs($mainTable);
+    my ($mainTag, $ns, $tag);
     foreach $mainTag (keys %$mainTable) {
         my $mainInfo = $mainTable->{$mainTag};
         next unless ref $mainInfo eq 'HASH' and $mainInfo->{SubDirectory};
-        my $table = Image::ExifTool::GetTagTable($mainInfo->{SubDirectory}->{TagTable});
-        # add new namespace if NAMESPACE is ns/uri pair
-        if (ref $$table{NAMESPACE} eq 'ARRAY') {
-            my $ns = $table->{NAMESPACE}->[0];
-            $nsURI{$ns} = $table->{NAMESPACE}->[1];
-            $$table{NAMESPACE} = $ns;
-        }
+        my $table = GetTagTable($mainInfo->{SubDirectory}->{TagTable});
         $$table{WRITE_PROC} = \&WriteXMP;   # set WRITE_PROC for all tables
-        Image::ExifTool::GenerateTagIDs($table);
-        my $tag;
+        GenerateTagIDs($table);
         $table->{CHECK_PROC} = \&CheckXMP; # add our write check routine
-        foreach $tag (Image::ExifTool::TagTableKeys($table)) {
+        foreach $tag (TagTableKeys($table)) {
             my $tagInfo = $$table{$tag};
             next unless ref $tagInfo eq 'HASH';
             # must set PropertyPath now for all tags that are Struct elements
@@ -224,11 +289,43 @@ my $pktClose =  "<?xpacket end='w'?>";
             $$tagInfo{PrintConvInv} = '$val' unless $$tagInfo{PrintConvInv};
             $$tagInfo{ValueConvInv} = '$val' unless $$tagInfo{ValueConvInv};
         }
+        # add new namespace if NAMESPACE is ns/uri pair
+        next unless ref $$table{NAMESPACE};
+        my $nsRef = $$table{NAMESPACE};
+        # recognize as either a list or hash
+        if (ref $nsRef eq 'ARRAY') {
+            $ns = $$nsRef[0];
+            $nsURI{$ns} = $$nsRef[1];
+        } else { # must be a hash
+            ($ns) = keys %$nsRef;
+            $nsURI{$ns} = $$nsRef{$ns};
+        }
+        $$table{NAMESPACE} = $ns;
     }
 }
 
 #------------------------------------------------------------------------------
-# check XMP values for validity and format accordingly
+# Validate XMP packet and set read or read/write mode
+# Inputs: 0) XMP data reference, 1) 'r' = read only, 'w' or undef = read/write
+# Returns: true if XMP is good (and adds packet header/trailer if necessary)
+sub ValidateXMP($;$)
+{
+    my ($xmpPt, $mode) = @_;
+    unless ($$xmpPt =~ /^\0*<\0*\?\0*x\0*p\0*a\0*c\0*k\0*e\0*t/) {
+        return '' unless $$xmpPt =~ /^<x:xmpmeta/;
+        # add required xpacket header/trailer
+        $$xmpPt = $pktOpen . $$xmpPt . $pktCloseW;
+    }
+    $mode = 'w' unless $mode;
+    my $end = substr($$xmpPt, -32, 32);
+    # check for proper xpacket trailer and set r/w mode if necessary
+    return '' unless $end =~ s/(e\0*n\0*d\0*=\0*['"]\0*)([rw])(\0*['"]\0*\?\0*>)/$1$mode$3/;
+    substr($$xmpPt, -32, 32) = $end if $2 ne $mode;
+    return 1;
+}
+
+#------------------------------------------------------------------------------
+# Check XMP values for validity and format accordingly
 # Inputs: 0) ExifTool object reference, 1) tagInfo hash reference,
 #         2) raw value reference
 # Returns: error string or undef (and may change value) on success
@@ -240,16 +337,19 @@ sub CheckXMP($$$)
     return undef unless $format and $format ne 'string';
     if ($format eq 'rational' or $format eq 'real') {
         # make sure the value is a valid floating point number
-        unless ($$valPtr =~ /^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/) {
-            return 'Not a floating point number';
-        }
+        Image::ExifTool::IsFloat($$valPtr) or return 'Not a floating point number';
         if ($format eq 'rational') {
             $$valPtr = join('/', Image::ExifTool::Rationalize($$valPtr));
         }
     } elsif ($format eq 'integer') {
         # make sure the value is integer
-        return 'Not an integer' unless $$valPtr =~ /^[+-]?\d+$/;
-        $$valPtr = int($$valPtr);
+        if (Image::ExifTool::IsInt($$valPtr)) {
+            # no conversion required (converting to 'int' would remove leading '+')
+        } elsif (Image::ExifTool::IsHex($$valPtr)) {
+            $$valPtr = hex($$valPtr);
+        } else {
+            return 'Not an integer';
+        }
     } elsif ($format eq 'date') {
         if ($$valPtr =~ /(\d{4}):(\d{2}):(\d{2}) (\d{2}:\d{2}:\d{2})(.*)/) {
             my ($y, $m, $d, $t, $tz) = ($1, $2, $3, $4, $5);
@@ -275,6 +375,9 @@ sub CheckXMP($$$)
         } else {
             $$valPtr = 'True';
         }
+    } elsif ($format eq '1') {
+        # this is the entire XMP data block
+        return 'Invalid XMP data' unless ValidateXMP($valPtr);
     } else {
         return "Unknown XMP format: $format";
     }
@@ -348,86 +451,116 @@ sub SetPropertyPath($$;$$)
 }
 
 #------------------------------------------------------------------------------
-# Capture shorthand XMP properties
-# Inputs: 0) ExifTool object reference, 1) Pointer to tag table
-#         2) reference to array of XMP property path (last is current property)
-#         3) reference to hash of property attributes
-#         4) true to set error on unrecognized attributes
-sub CaptureShorthand($$$;$)
+# Save XMP property name/value for rewriting
+# Inputs: 0) ExifTool object reference
+#         1) reference to array of XMP property path (last is current property)
+#         2) property value, 3) optional reference to hash of property attributes
+sub CaptureXMP($$$;$)
 {
-    my ($exifTool, $propList, $attrs, $setError) = @_;
-    my $attr;
-    my @attrList = keys %$attrs;
-    foreach $attr (@attrList) {
-        if ($attr =~ /^xmlns:(.*)/) {
-            # remember all namespaces (except x which we open ourselves)
-            my $ns = $1;
-            my $nsUsed = $exifTool->{XMP_NS};
-            unless ($ns eq 'x' or $ns eq 'iX' or defined $$nsUsed{$ns}) {
-                $$nsUsed{$ns} = $$attrs{$attr};
-            }
-        } elsif (not $recognizedAttrs{$attr}) {
-            # check for RDF shorthand format
-            my ($ns,$name);
-            if ($attr =~ /(.*):(.*)/) {
-                $ns = $1;
-                $name = $2;
+    my ($exifTool, $propList, $val, $attrs) = @_;
+    return unless defined $val and @$propList > 2;
+    if ($$propList[0] =~ /^x:x(a|m)pmeta$/ and
+        $$propList[1] eq 'rdf:RDF' and
+        $$propList[2] =~ /$rdfDesc( |$)/)
+    {
+        # no properties to save yet if this is just the description
+        return unless @$propList > 3;
+        # save information about this property
+        my $capture = $exifTool->{XMP_CAPTURE};
+        my $path = join('/', @$propList[3..$#$propList]);
+        if (defined $$capture{$path}) {
+            $exifTool->{XMP_ERROR} = "Duplicate XMP property: $path";
+        } else {
+            $$capture{$path} = [$val, $attrs || { }];
+        }
+    } else {
+        $exifTool->{XMP_ERROR} = 'Improperly enclosed XMP property: ' . join('/',@$propList);
+    }
+}
+
+#------------------------------------------------------------------------------
+# Save information about resource containing blank node with nodeID
+# Inputs: 0) reference to blank node information hash
+#         1) reference to property list
+#         2) property value
+#         3) [optional] reference to attribute hash
+# Notes: This routine and ProcessBlankInfo() are also used for reading information, but
+#        are uncommon so are put in this file to reduce compile time for the common case
+sub SaveBlankInfo($$$;$)
+{
+    my ($blankInfo, $propListPt, $val, $attrs) = @_;
+    
+    my $propPath = join '/', @$propListPt;
+    my @ids = ($propPath =~ m{ #([^ /]*)}g);
+    my $id;
+    # split the property path at each nodeID
+    foreach $id (@ids) {
+        my ($pre, $prop, $post) = ($propPath =~ m{^(.*?)/([^/]*) #$id((/.*)?)$});
+        defined $pre or warn("internal error parsing nodeID's"), next;
+        # the element with the nodeID should be in the path prefix for subject
+        # nodes and the path suffix for object nodes
+        unless ($prop eq $rdfDesc) {
+            if ($post) {
+                $post = "/$prop$post";
             } else {
-                my $prop = $$propList[$#$propList];
-                $ns = $1 if $prop =~ /(.*):/;   # take namespace from enclosing property
-                $name = $attr;
-            }
-            if ($ns and ($Image::ExifTool::XMP::Main{$ns} or ($xlatNamespace{$ns} and
-                $Image::ExifTool::XMP::Main{$xlatNamespace{$ns}})))
-            {
-                # save this shorthand property
-                push @$propList, "$ns:$name";
-                CaptureXMP($exifTool, $propList, $$attrs{$attr});
-                pop @$propList;
-                # remove this attribute from list since we handled it
-                delete $$attrs{$attr};
-            } elsif ($setError) {
-                $exifTool->{XMP_ERROR} = "Can't yet handle XMP attribute '$attr'";
+                $pre = "$pre/$prop";
             }
         }
+        $blankInfo->{Prop}->{$id}->{Pre}->{$pre} = 1;
+        if ((defined $post and length $post) or (defined $val and length $val)) {
+            # save the property value and attributes for each unique path suffix
+            $blankInfo->{Prop}->{$id}->{Post}->{$post} = [ $val, $attrs, $propPath ];
+        }
     }
 }
 
 #------------------------------------------------------------------------------
-# Save XMP property name/value for rewriting
-# Inputs: 0) ExifTool object reference, 1) Pointer to tag table
-#         2) reference to array of XMP property path (last is current property)
-#         3) property value, 4) optional reference to hash of property attributes
-sub CaptureXMP($$$;$)
+# Process blank-node information
+# Inputs: 0) ExifTool object reference
+#         1) reference to tag table
+#         2) reference to blank node information hash
+#         3) flag set for writing
+sub ProcessBlankInfo($$$;$)
 {
-    my ($exifTool, $propList, $val, $attrs) = @_;
-    $attrs or $attrs = { };
-
-    if (defined $val) {
-        return unless @$propList > 2;
-        if ($$propList[0] =~ /^x:x(a|m)pmeta$/ and
-            $$propList[1] eq 'rdf:RDF' and
-            $$propList[2] eq $rdfDesc)
-        {
-            # capture any shorthand properties in the attribute list
-            CaptureShorthand($exifTool, $propList, $attrs);
-            # no properties to save yet if this is just the description
-            return unless @$propList > 3;
-            # save information about this property
-            my $capture = $exifTool->{XMP_CAPTURE};
-            my $path = join('/', @$propList[3..$#$propList]);
-            if (defined $$capture{$path}) {
-                $exifTool->{XMP_ERROR} = "Duplicate XMP property: $path";
-            } else {
-                $$capture{$path} = [$val, $attrs];
+    my ($exifTool, $tagTablePtr, $blankInfo, $isWriting) = @_;
+    $exifTool->VPrint(1, "  [Elements with nodeID set:]\n") unless $isWriting;
+    my ($id, $pre, $post);
+    # handle each nodeID separately
+    foreach $id (sort keys %{$$blankInfo{Prop}}) {
+        my $path = $blankInfo->{Prop}->{$id};
+        # flag all resource names so we can warn later if some are unused
+        my %unused;
+        foreach $post (keys %{$path->{Post}}) {
+            $unused{$post} = 1;
+        }
+        # combine property paths for all possible paths through this node
+        foreach $pre (sort keys %{$path->{Pre}}) {
+            # there will be no description for the object of a blank node
+            next unless $pre =~ m{/$rdfDesc/};
+            foreach $post (sort keys %{$path->{Post}}) {
+                my @propList = split m{/}, "$pre$post";
+                my ($val, $attrs) = @{$path->{Post}->{$post}};
+                if ($isWriting) {
+                    CaptureXMP($exifTool, \@propList, $val, $attrs);
+                } else {
+                    FoundXMP($exifTool, $tagTablePtr, \@propList, $val);
+                }
+                delete $unused{$post};
             }
-        } else {
-            $exifTool->{XMP_ERROR} = 'Improperly enclosed XMP property: ' . join('/',@$propList);
+        }
+        # save information from unused properties (if RDF is malformed like f-spot output)
+        if (%unused) {
+            $exifTool->Options('Verbose') and $exifTool->Warn("An XMP resource is about nothing");
+            foreach $post (sort keys %unused) {
+                my ($val, $attrs, $propPath) = @{$path->{Post}->{$post}};
+                my @propList = split m{/}, $propPath;
+                if ($isWriting) {
+                    CaptureXMP($exifTool, \@propList, $val, $attrs);
+                } else {
+                    FoundXMP($exifTool, $tagTablePtr, \@propList, $val);
+                }
+            }
         }
-    } else {
-        # this property has sub-properties, so just handle the attributes
-        # (set an error on any unrecognized attributes here, because they will be lost)
-        CaptureShorthand($exifTool, $propList, $attrs, 1);
     }
 }
 
@@ -464,19 +597,21 @@ sub ConformPathToNamespace($$)
 #------------------------------------------------------------------------------
 # Write XMP information
 # Inputs: 0) ExifTool object reference, 1) source dirInfo reference,
-#         2) tag table reference
-# Returns: new XMP data (may be empty if no XMP data) or undef on error
-# Notes: Increments ExifTool CHANGED flag for each tag changed
-sub WriteXMP($$$)
+#         2) [optional] tag table reference
+# Returns: with tag table: new XMP data (may be empty if no XMP data) or undef on error
+#          without tag table: 1 on success, 0 if not valid XMP file, -1 on write error
+# Notes: May set dirInfo InPlace flag to rewrite with specified DirLen
+#        May set dirInfo ReadOnly flag to write as read-only XMP ('r' mode and no padding)
+sub WriteXMP($$;$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
     $exifTool or return 1;    # allow dummy access to autoload this package
-    $tagTablePtr or return undef;
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart} || 0;
-    my (%capture, %nsUsed, $xmpErr);
+    my (%capture, %nsUsed, $xmpErr, $uuid);
     my $changed = 0;
-    my $verbose = $exifTool->Options('Verbose');
+    my $xmpFile = (not $tagTablePtr);   # this is an XMP data file if no $tagTablePtr
+    my $preferred = $xmpFile;   # write XMP as preferred if this is an XMP file
 #
 # extract existing XMP information into %capture hash
 #
@@ -489,26 +624,36 @@ sub WriteXMP($$$)
     $exifTool->{XMP_CAPTURE} = \%capture;
     $exifTool->{XMP_NS} = \%nsUsed;
 
-    if ($dataPt) {
+    if ($dataPt or $xmpFile) {
         delete $exifTool->{XMP_ERROR};
+        delete $exifTool->{XMP_UUID};
         # extract all existing XMP information (to the XMP_CAPTURE hash)
         my $success = ProcessXMP($exifTool, $dirInfo, $tagTablePtr);
         # don't continue if there is nothing to parse or if we had a parsing error
         unless ($success and not $exifTool->{XMP_ERROR}) {
-            if ($exifTool->{XMP_ERROR}) {
-                $exifTool->Warn($exifTool->{XMP_ERROR}, 1);
+            my $err = $exifTool->{XMP_ERROR} || 'Error parsing XMP';
+            # may ignore this error only if we were successful
+            if ($xmpFile) {
+                my $raf = $$dirInfo{RAF};
+                # allow empty XMP data so we can create something from nothing
+                if ($success or not $raf->Seek(0,2) or $raf->Tell()) {
+                    if ($exifTool->Error($err, $success)) {
+                        delete $exifTool->{XMP_CAPTURE};
+                        return 0;
+                    }
+                }
             } else {
-                $exifTool->Warn('Error parsing XMP', 1);
-            }
-            unless ($success and $exifTool->Options('IgnoreMinorErrors')) {
-                delete $exifTool->{XMP_CAPTURE};
-                return undef;
+                if ($exifTool->Warn($err, $success)) {
+                    delete $exifTool->{XMP_CAPTURE};
+                    return undef;
+                }
             }
         }
+        $uuid = $exifTool->{XMP_UUID} || '';
         delete $exifTool->{XMP_ERROR};
+        delete $exifTool->{XMP_UUID};
     } else {
-        my $emptyData = '';
-        $dataPt = \$emptyData;
+        $uuid = '';
     }
 #
 # add, delete or change information as specified
@@ -530,13 +675,12 @@ sub WriteXMP($$$)
         # find existing property
         my $capList = $capture{$path};
         my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
-        my @newValues = Image::ExifTool::GetNewValues($newValueHash);
+        my $overwrite = Image::ExifTool::IsOverwriting($newValueHash);
         my %attrs;
         # delete existing entry if necessary
         if ($capList) {
             # take attributes from old values if they exist
             %attrs = %{$capList->[1]};
-            my $overwrite = Image::ExifTool::IsOverwriting($newValueHash);
             if ($overwrite) {
                 my ($delPath, @matchingPaths);
                 # check to see if this is an indexed list item
@@ -552,8 +696,9 @@ sub WriteXMP($$$)
                     if ($overwrite < 0) {
                         # only overwrite specific values
                         next unless Image::ExifTool::IsOverwriting($newValueHash, $val);
+                        $overwrite = 1;
                     }
-                    $verbose > 1 and print "    - XMP:$$tagInfo{Name} = '$val'\n";
+                    $exifTool->VPrint(1, "    - XMP:$$tagInfo{Name} = '$val'\n");
                     # save attributes and path from this deleted property
                     # so we can replace it exactly
                     %attrs = %$attrs;
@@ -586,9 +731,13 @@ sub WriteXMP($$$)
                 }
             }
         }
-        next unless @newValues; # done if no new values specified
+        next unless $overwrite > 0;
+        # get list of new values (done if no new values specified)
+        my @newValues = Image::ExifTool::GetNewValues($newValueHash) or next;
         # don't add new tag if it didn't exist before unless specified
-        next unless $capList or Image::ExifTool::IsCreating($newValueHash);
+        # (or unless this is an XMP data file and we aren't avoiding this tag)
+        next unless $capList or Image::ExifTool::IsCreating($newValueHash) or
+            ($preferred and not $$tagInfo{Avoid});
 
         # set default language attribute for lang-alt lists
         # (currently on support changing the default language)
@@ -598,7 +747,7 @@ sub WriteXMP($$$)
         for (;;) {
             my $newValue = EscapeHTML(shift @newValues);
             $capture{$path} = [ $newValue, \%attrs ];
-            $verbose > 1 and print "    + XMP:$$tagInfo{Name} = '$newValue'\n";
+            $exifTool->VPrint(1, "    + XMP:$$tagInfo{Name} = '$newValue'\n");
             ++$changed;
             last unless @newValues;
             $path =~ m/ (\d{3})/g or warn("Internal error: no list index!\n"), next;
@@ -616,12 +765,25 @@ sub WriteXMP($$$)
     delete $exifTool->{XMP_NS};
 
     # return now if we didn't change anything
-    return undef unless $changed;
+    unless ($changed) {
+        return undef unless $xmpFile;   # just rewrite original XMP
+        # get DataPt again because it may have been set by ProcessXMP
+        $dataPt = $$dirInfo{DataPt};
+        unless (defined $dataPt) {
+            $exifTool->Error("Nothing to write");
+            return 1;
+        }
+        return 1 if Write($$dirInfo{OutFile}, $$dataPt);
+        return -1;
+    }
 #
 # write out the new XMP information
 #
     # start writing the XMP data
-    my $newData = $pktOpen . $xmpOpen . $rdfOpen;
+    my $newData = '';
+    $newData .= $pktOpen unless $$exifTool{XMP_NO_XPACKET};
+    $newData .= $xmlOpen if $$exifTool{XMP_IS_XML};
+    $newData .= $xmpOpen . $rdfOpen;
 
     # initialize current property path list
     my @curPropList;
@@ -665,7 +827,7 @@ sub WriteXMP($$$)
             # open the new description
             $prop = $rdfDesc;
             %nsCur = %nsNew;            # save current namespaces
-            $newData .= "\n <$prop about=''";
+            $newData .= "\n <$prop rdf:about='$uuid'";
             foreach (sort keys %nsCur) {
                 $newData .= "\n  xmlns:$_='$nsCur{$_}'";
             }
@@ -706,28 +868,56 @@ sub WriteXMP($$$)
         $prop =~ s/ .*//;   # remove list index if it exists
         $newData .= (' ' x scalar(@curPropList)) . " </$prop>\n";
     }
-    $newData .= $rdfClose . $xmpClose;
-    # (the XMP standard recommends writing 2k-4k of white space before the
-    # packet trailer, with a newline every 100 characters)
-    $newData .= ((' ' x 100) . "\n") x 24 unless $exifTool->Options('Compact');
-    $newData .= $pktClose;
 #
-# clean up and return our data
+# clean up, close out the XMP, and return our data
 #
     # remove the ExifTool members we created
     delete $exifTool->{XMP_CAPTURE};
     delete $exifTool->{XMP_NS};
 
+    $newData .= $rdfClose . $xmpClose;
+
+    # (the XMP standard recommends writing 2k-4k of white space before the
+    # packet trailer, with a newline every 100 characters)
+    unless ($$exifTool{XMP_NO_XPACKET}) {
+        my $pad = (' ' x 100) . "\n";
+        if ($$dirInfo{InPlace}) {
+            # pad to specified DirLen
+            my $dirLen = $$dirInfo{DirLen} || length $$dataPt;
+            my $len = length($newData) + length($pktCloseW);
+            if ($len > $dirLen) {
+                $exifTool->Warn('Not enough room to edit XMP in place');
+                return undef;
+            }
+            my $num = int(($dirLen - $len) / length($pad));
+            if ($num) {
+                $newData .= $pad x $num;
+                $len += length($pad) * $num;
+            }
+            $len < $dirLen and $newData .= (' ' x ($dirLen - $len - 1)) . "\n";
+        } elsif (not $exifTool->Options('Compact') and
+                 not $xmpFile and not $$dirInfo{ReadOnly})
+        {
+            $newData .= $pad x 24;
+        }
+        $newData .= ($$dirInfo{ReadOnly} ? $pktCloseR : $pktCloseW);
+    }
     # return empty data if no properties exist
-    $newData = '' unless %capture;
+    $newData = '' unless %capture or $$dirInfo{InPlace};
 
     if ($xmpErr) {
+        if ($xmpFile) {
+            $exifTool->Error($xmpErr);
+            return -1;
+        }
         $exifTool->Warn($xmpErr);
         return undef;
     }
     $exifTool->{CHANGED} += $changed;
     $debug > 1 and $newData and print $newData,"\n";
-    return $newData;
+    return $newData unless $xmpFile;
+    return 1 if Write($$dirInfo{OutFile}, $newData);
+    return -1;
 }
 
 
@@ -737,7 +927,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::WriteXMP.pl - Routines for writing XMP metadata
+Image::ExifTool::WriteXMP.pl - Write XMP meta information
 
 =head1 SYNOPSIS
 
@@ -749,10 +939,10 @@ This file contains routines to write XMP metadata.
 
 =head1 AUTHOR
 
-Copyright 2003-2005, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This library is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
